@@ -24,40 +24,98 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "cfg.h"
+#include <math.h>
+
+#include <cfg.h>
 #include <plog.h>
-#include "psensor.h"
+#include <psensor.h>
 
 /* horizontal padding */
-#define GRAPH_H_PADDING 4
+const int GRAPH_H_PADDING = 4;
 /* vertical padding */
-#define GRAPH_V_PADDING 4
+const int GRAPH_V_PADDING = 4;
 
-static time_t get_graph_end_time_s()
+bool is_smooth_curves_enabled;
+
+struct graph_info {
+	/* Horizontal position of the central area (curves) */
+	int g_xoff;
+	/* Vertical position of the central area (curves) */
+	int g_yoff;
+
+	/* Width of the central area (curves) */
+	int g_width;
+	/* Height of the central area (curves) */
+	int g_height;
+
+	/* Height of the drawing canvas */
+	int height;
+	/* Width of the drawing canvas */
+	int width;
+
+	/* Background color of the current desktop theme */
+	GdkRGBA theme_bg_color;
+	/* Foreground color of the current desktop theme */
+	GdkRGBA theme_fg_color;
+};
+
+/* Return the end time of the graph i.e. the more recent measure.  If
+ * no measure are available, return 0.
+ * If Bezier curves are used return the measure n-3 to avoid to
+ * display a part of the curve outside the graph area.
+ */
+static time_t get_graph_end_time_s(struct psensor **sensors)
 {
-	struct timeval tv;
+	time_t ret, t;
+	struct psensor *s;
+	struct measure *measures;
+	int i, n;
 
-	if (gettimeofday(&tv, NULL) == 0)
-		return tv.tv_sec;
-	else
-		return 0;
+	ret = 0;
+	while (*sensors) {
+		s = *sensors;
+		measures = s->measures;
+
+		if (is_smooth_curves_enabled)
+			n = 2;
+		else
+			n = 0;
+
+		for (i = s->values_max_length - 1; i >= 0; i--) {
+			if (measures[i].value != UNKNOWN_DBL_VALUE) {
+				if (!n) {
+					t = measures[i].time.tv_sec;
+
+					if (t > ret) {
+						ret = t;
+						break;
+					}
+				} else {
+					n--;
+				}
+			}
+			i--;
+		}
+
+		sensors++;
+	}
+
+	return ret;
 }
 
-static time_t get_graph_begin_time_s(struct config *cfg)
+static time_t get_graph_begin_time_s(struct config *cfg, time_t etime)
 {
-	int ct;
-
-	ct = get_graph_end_time_s();
-
-	if (!ct)
+	if (!etime)
 		return 0;
 
-	return ct - cfg->graph_monitoring_duration * 60;
+	return etime - cfg->graph_monitoring_duration * 60;
 }
 
-static int compute_y(double value, double min, double max, int height, int off)
+static double
+compute_y(double value, double min, double max, int height, int off)
 {
 	double t = value - min;
+
 	return height - ((double)height * (t / (max - min))) + off;
 }
 
@@ -76,52 +134,74 @@ static char *time_to_str(time_t s)
 	return str;
 }
 
+static void draw_left_region(cairo_t *cr, struct graph_info *info)
+{
+	cairo_set_source_rgb(cr,
+			     info->theme_bg_color.red,
+			     info->theme_bg_color.green,
+			     info->theme_bg_color.blue);
+
+	cairo_rectangle(cr, 0, 0, info->g_xoff, info->height);
+	cairo_fill(cr);
+}
+
+static void draw_right_region(cairo_t *cr, struct graph_info *info)
+{
+	cairo_set_source_rgb(cr,
+			     info->theme_bg_color.red,
+			     info->theme_bg_color.green,
+			     info->theme_bg_color.blue);
+
+
+	cairo_rectangle(cr,
+			info->g_xoff + info->g_width,
+			0,
+			info->g_xoff + info->g_width + GRAPH_H_PADDING,
+			info->height);
+	cairo_fill(cr);
+}
+
 static void
 draw_graph_background(cairo_t *cr,
-		      int g_xoff, int g_yoff,
-		      int g_width, int g_height,
-		      int width, int height, struct config *config,
-		      GtkWidget *widget,
-		      GtkWidget *window)
+		      struct config *config,
+		      struct graph_info *info)
 {
-	GtkStyleContext *style_ctx;
 	struct color *bgcolor;
-	GdkRGBA rgba;
 
 	bgcolor = config->graph_bgcolor;
 
-	style_ctx = gtk_widget_get_style_context(window);
-	gtk_style_context_get_background_color(style_ctx,
-					       GTK_STATE_FLAG_NORMAL,
-					       &rgba);
-
 	if (config->alpha_channel_enabled)
 		cairo_set_source_rgba(cr,
-				      rgba.red,
-				      rgba.green,
-				      rgba.blue,
+				      info->theme_bg_color.red,
+				      info->theme_bg_color.green,
+				      info->theme_bg_color.blue,
 				      config->graph_bg_alpha);
 	else
 		cairo_set_source_rgb(cr,
-				     rgba.red,
-				     rgba.green,
-				     rgba.blue);
+				     info->theme_bg_color.red,
+				     info->theme_bg_color.green,
+				     info->theme_bg_color.blue);
 
-	cairo_rectangle(cr, 0, 0, width, height);
+	cairo_rectangle(cr, info->g_xoff, 0, info->g_width, info->height);
 	cairo_fill(cr);
+
 	if (config->alpha_channel_enabled)
 		cairo_set_source_rgba(cr,
-				      bgcolor->f_red,
-				      bgcolor->f_green,
-				      bgcolor->f_blue,
+				      bgcolor->red,
+				      bgcolor->green,
+				      bgcolor->blue,
 				      config->graph_bg_alpha);
 	else
 		cairo_set_source_rgb(cr,
-				     bgcolor->f_red,
-				     bgcolor->f_green,
-				     bgcolor->f_blue);
+				     bgcolor->red,
+				     bgcolor->green,
+				     bgcolor->blue);
 
-	cairo_rectangle(cr, g_xoff, g_yoff, g_width, g_height);
+	cairo_rectangle(cr,
+			info->g_xoff,
+			info->g_yoff,
+			info->g_width,
+			info->g_height);
 	cairo_fill(cr);
 }
 
@@ -133,60 +213,173 @@ static double dashes[] = {
 static int ndash = sizeof(dashes) / sizeof(dashes[0]);
 
 static void draw_background_lines(cairo_t *cr,
-				  struct color *color,
-				  int g_width, int g_height,
-				  int g_xoff, int g_yoff,
-				  int min, int max)
+				  int min, int max,
+				  struct config *config,
+				  struct graph_info *info)
 {
 	int i;
+	double x, y;
+	struct color *color;
+
+	color = config->graph_fgcolor;
 
 	/* draw background lines */
 	cairo_set_line_width(cr, 1);
 	cairo_set_dash(cr, dashes, ndash, 0);
-	cairo_set_source_rgb(cr,
-			     color->f_red, color->f_green, color->f_blue);
+	cairo_set_source_rgb(cr, color->red, color->green, color->blue);
 
 	/* vertical lines representing time steps */
 	for (i = 0; i <= 5; i++) {
-		int x = i * (g_width / 5) + g_xoff;
-		cairo_move_to(cr, x, g_yoff);
-		cairo_line_to(cr, x, g_yoff + g_height);
-		cairo_stroke(cr);
+		x = i * ((double)info->g_width / 5) + info->g_xoff;
+		cairo_move_to(cr, x, info->g_yoff);
+		cairo_line_to(cr, x, info->g_yoff + info->g_height);
 	}
 
 	/* horizontal lines draws a line for each 10C step */
 	for (i = min; i < max; i++) {
 		if (i % 10 == 0) {
-			int y = compute_y(i, min, max, g_height, g_yoff);
+			y = compute_y(i,
+				      min,
+				      max,
+				      info->g_height,
+				      info->g_yoff);
 
-			cairo_move_to(cr, g_xoff, y);
-			cairo_line_to(cr, g_xoff + g_width, y);
-			cairo_stroke(cr);
+			cairo_move_to(cr, info->g_xoff, y);
+			cairo_line_to(cr, info->g_xoff + info->g_width, y);
 		}
 	}
 
+	cairo_stroke(cr);
+
 	/* back to normal line style */
 	cairo_set_dash(cr, 0, 0, 0);
 }
 
+/* Keys: sensor identifier.
+ *
+ * Values: array of time_t. Each time_t is corresponding to a sensor
+ * measure which has been used as the start point of a Bezier curve.
+ */
+static GHashTable *times;
+
+static void draw_sensor_smooth_curve(struct psensor *s,
+				     cairo_t *cr,
+				     double min,
+				     double max,
+				     int bt,
+				     int et,
+				     struct graph_info *info)
+{
+	int i, dt, vdt, j, k, found;
+	double x[4], y[4], v;
+	time_t t, t0, *stimes;
+
+	if (!times)
+		times = g_hash_table_new_full(g_str_hash,
+					      g_str_equal,
+					      free,
+					      free);
+
+	stimes = g_hash_table_lookup(times, s->id);
+
+	cairo_set_source_rgb(cr,
+			     s->color->red,
+			     s->color->green,
+			     s->color->blue);
+
+	/* search the index of the first measure used as a start point
+	 * of a Bezier curve. The start and end points of the Bezier
+	 * curves must be preserved to ensure the same overall shape
+	 * of the graph. */
+	i = 0;
+	if (stimes) {
+		while (i < s->values_max_length) {
+			t = s->measures[i].time.tv_sec;
+			v = s->measures[i].value;
+
+			found = 0;
+			if (v != UNKNOWN_DBL_VALUE && t) {
+				k = 0;
+				while (stimes[k]) {
+					if (t == stimes[k]) {
+						found = 1;
+						break;
+					}
+					k++;
+				}
+			}
+
+			if (found)
+				break;
+
+			i++;
+		}
+	}
+
+	stimes = malloc((s->values_max_length + 1) * sizeof(time_t));
+	memset(stimes, 0, (s->values_max_length + 1) * sizeof(time_t));
+	g_hash_table_insert(times, strdup(s->id), stimes);
+
+	if (i == s->values_max_length)
+		i = 0;
+
+	k = 0;
+	dt = et - bt;
+	while (i < s->values_max_length) {
+		j = 0;
+		t = 0;
+		while (i < s->values_max_length && j < 4) {
+			t = s->measures[i].time.tv_sec;
+			v = s->measures[i].value;
+
+			if (v == UNKNOWN_DBL_VALUE || !t) {
+				i++;
+				continue;
+			}
+
+			vdt = t - bt;
+
+			x[0 + j] = ((double)vdt * info->g_width)
+				/ dt + info->g_xoff;
+			y[0 + j] = compute_y(v,
+					     min,
+					     max,
+					     info->g_height,
+					     info->g_yoff);
+
+			if (j == 0)
+				t0 = t;
+
+			i++;
+			j++;
+		}
+
+		if (j == 4) {
+			cairo_move_to(cr, x[0], y[0]);
+			cairo_curve_to(cr, x[1], y[1], x[2], y[3], x[3], y[3]);
+			stimes[k++] = t0;
+			i--;
+		}
+	}
+
+	cairo_stroke(cr);
+}
+
 static void draw_sensor_curve(struct psensor *s,
 			      cairo_t *cr,
 			      double min,
 			      double max,
 			      int bt,
 			      int et,
-			      int g_width,
-			      int g_height,
-			      int g_xoff,
-			      int g_yoff)
+			      struct graph_info *info)
 {
-	int first, i, x, y, t, dt, vdt;
-	double v;
+	int first, i, t, dt, vdt;
+	double v, x, y;
 
 	cairo_set_source_rgb(cr,
-			     s->color->f_red,
-			     s->color->f_green,
-			     s->color->f_blue);
+			     s->color->red,
+			     s->color->green,
+			     s->color->blue);
 
 	dt = et - bt;
 	first = 1;
@@ -198,12 +391,10 @@ static void draw_sensor_curve(struct psensor *s,
 			continue;
 
 		vdt = t - bt;
-		if (vdt < 0)
-			continue;
 
-		x = vdt * g_width / dt + g_xoff;
+		x = ((double)vdt * info->g_width) / dt + info->g_xoff;
 
-		y = compute_y(v, min, max, g_height, g_yoff);
+		y = compute_y(v, min, max, info->g_height, info->g_yoff);
 
 		if (first) {
 			cairo_move_to(cr, x, y);
@@ -240,12 +431,11 @@ graph_update(struct psensor **sensors,
 	     struct config *config,
 	     GtkWidget *window)
 {
-	struct color *fgcolor = config->graph_fgcolor;
 	int et, bt, width, height, g_width, g_height;
-	double min_rpm, max_rpm, mint, maxt;
+	double min_rpm, max_rpm, mint, maxt, min, max;
 	char *strmin, *strmax;
 	/* horizontal and vertical offset of the graph */
-	int g_xoff, g_yoff, no_graphs, min, max;
+	int g_xoff, g_yoff, no_graphs;
 	cairo_surface_t *cst;
 	cairo_t *cr, *cr_pixmap;
 	char *str_btime, *str_etime;
@@ -253,10 +443,10 @@ graph_update(struct psensor **sensors,
 	struct psensor **sensor_cur, **enabled_sensors;
 	GtkAllocation galloc;
 	GtkStyleContext *style_ctx;
-	GdkRGBA rgba;
+	struct graph_info info;
 
 	if (!gtk_widget_is_drawable(w_graph))
-		return ;
+		return;
 
 	enabled_sensors = psensor_list_filter_graph_enabled(sensors);
 
@@ -273,12 +463,18 @@ graph_update(struct psensor **sensors,
 				      maxt,
 				      config->temperature_unit == CELSIUS);
 
-	str_btime = time_to_str(get_graph_begin_time_s(config));
-	str_etime = time_to_str(get_graph_end_time_s());
+	et = get_graph_end_time_s(enabled_sensors);
+	bt = get_graph_begin_time_s(config, et);
+
+	str_btime = time_to_str(bt);
+	str_etime = time_to_str(et);
 
 	gtk_widget_get_allocation(w_graph, &galloc);
 	width = galloc.width;
+	info.width = galloc.width;
 	height = galloc.height;
+	info.height = height;
+
 
 	cst = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
 	cr = cairo_create(cst);
@@ -295,6 +491,7 @@ graph_update(struct psensor **sensors,
 	cairo_text_extents(cr, strmin, &te_min);
 
 	g_yoff = GRAPH_V_PADDING;
+	info.g_yoff = g_yoff;
 
 	g_height = height - GRAPH_V_PADDING;
 	if (te_etime.height > te_btime.height)
@@ -302,23 +499,33 @@ graph_update(struct psensor **sensors,
 	else
 		g_height -= GRAPH_V_PADDING + te_btime.height + GRAPH_V_PADDING;
 
+	info.g_height = g_height;
+
 	if (te_min.width > te_max.width)
 		g_xoff = (2 * GRAPH_H_PADDING) + te_max.width;
 	else
 		g_xoff = (2 * GRAPH_H_PADDING) + te_min.width;
 
-	g_width = width - g_xoff - GRAPH_H_PADDING;
+	info.g_xoff = g_xoff;
 
-	draw_graph_background(cr,
-			      g_xoff, g_yoff, g_width, g_height,
-			      width, height, config,
-			      w_graph,
-			      window);
-
-	/** Set the color for text drawing */
 	style_ctx = gtk_widget_get_style_context(window);
-	gtk_style_context_get_color(style_ctx, GTK_STATE_FLAG_NORMAL, &rgba);
-	cairo_set_source_rgb(cr, rgba.red, rgba.green, rgba.blue);
+	gtk_style_context_get_background_color(style_ctx,
+					       GTK_STATE_FLAG_NORMAL,
+					       &info.theme_bg_color);
+	gtk_style_context_get_color(style_ctx,
+				    GTK_STATE_FLAG_NORMAL,
+				    &info.theme_fg_color);
+
+	g_width = width - g_xoff - GRAPH_H_PADDING;
+	info.g_width = g_width;
+
+	draw_graph_background(cr, config, &info);
+
+	/* Set the color for text drawing */
+	cairo_set_source_rgb(cr,
+			     info.theme_fg_color.red,
+			     info.theme_fg_color.green,
+			     info.theme_fg_color.blue);
 
 	/* draw graph begin time */
 	cairo_move_to(cr, g_xoff, height - GRAPH_V_PADDING);
@@ -332,25 +539,9 @@ graph_update(struct psensor **sensors,
 	cairo_show_text(cr, str_etime);
 	free(str_etime);
 
-	/* draw min and max temp */
-	cairo_move_to(cr, GRAPH_H_PADDING, te_max.height + GRAPH_V_PADDING);
-	cairo_show_text(cr, strmax);
-	free(strmax);
-
-	cairo_move_to(cr,
-		      GRAPH_H_PADDING, height - (te_min.height / 2) - g_yoff);
-	cairo_show_text(cr, strmin);
-	free(strmin);
-
-	draw_background_lines(cr, fgcolor,
-			      g_width, g_height,
-			      g_xoff, g_yoff,
-			      mint, maxt);
+	draw_background_lines(cr, mint, maxt, config, &info);
 
 	/* .. and finaly draws the temperature graphs */
-	bt = get_graph_begin_time_s(config);
-	et = get_graph_end_time_s();
-
 	if (bt && et) {
 		sensor_cur = enabled_sensors;
 
@@ -361,23 +552,28 @@ graph_update(struct psensor **sensors,
 			struct psensor *s = *sensor_cur;
 
 			no_graphs = 0;
-			if (is_fan_type(s->type)) {
+			if (s->type & SENSOR_TYPE_RPM) {
 				min = min_rpm;
 				max = max_rpm;
-			} else if (s->type & SENSOR_TYPE_CPU_USAGE) {
+			} else if (s->type & SENSOR_TYPE_PERCENT) {
 				min = 0;
 				max = get_max_value(enabled_sensors,
-						    SENSOR_TYPE_CPU_USAGE);
+						    SENSOR_TYPE_PERCENT);
 			} else {
 				min = mint;
 				max = maxt;
 			}
 
-			draw_sensor_curve(s, cr,
-					  min, max,
-					  bt, et,
-					  g_width, g_height,
-					  g_xoff, g_yoff);
+			if (is_smooth_curves_enabled)
+				draw_sensor_smooth_curve(s, cr,
+							 min, max,
+							 bt, et,
+							 &info);
+			else
+				draw_sensor_curve(s, cr,
+						  min, max,
+						  bt, et,
+						  &info);
 
 			sensor_cur++;
 		}
@@ -388,10 +584,27 @@ graph_update(struct psensor **sensors,
 						  g_height / 2);
 	}
 
+	draw_left_region(cr, &info);
+	draw_right_region(cr, &info);
+
+	/* draw min and max temp */
+	cairo_set_source_rgb(cr,
+			     info.theme_fg_color.red,
+			     info.theme_fg_color.green,
+			     info.theme_fg_color.blue);
+
+	cairo_move_to(cr, GRAPH_H_PADDING, te_max.height + GRAPH_V_PADDING);
+	cairo_show_text(cr, strmax);
+	free(strmax);
+
+	cairo_move_to(cr,
+		      GRAPH_H_PADDING, height - (te_min.height / 2) - g_yoff);
+	cairo_show_text(cr, strmin);
+	free(strmin);
+
 	cr_pixmap = gdk_cairo_create(gtk_widget_get_window(w_graph));
 
 	if (cr_pixmap) {
-
 		if (config->alpha_channel_enabled)
 			cairo_set_operator(cr_pixmap, CAIRO_OPERATOR_SOURCE);
 
@@ -405,3 +618,15 @@ graph_update(struct psensor **sensors,
 	cairo_surface_destroy(cst);
 	cairo_destroy(cr);
 }
+
+int compute_values_max_length(struct config *c)
+{
+	int n, duration, interval;
+
+	duration = c->graph_monitoring_duration * 60;
+	interval = c->sensor_update_interval;
+
+	n = 3 + ceil((((double)duration) / interval) + 0.5) + 3;
+
+	return n;
+}
