@@ -25,6 +25,8 @@
 #include <stdbool.h>
 #else
 #define bool char
+#define true 1
+#define false 0
 #endif
 
 #endif
