@@ -39,11 +39,14 @@
 
 #ifdef HAVE_GTOP
 #include "sysinfo.h"
-#include "cpu.h"
+#include <pgtop2.h>
 #endif
 
+#include <hdd.h>
+#include <lmsensor.h>
 #include <plog.h>
 #include "psensor_json.h"
+#include <pmutex.h>
 #include "url.h"
 #include "server.h"
 #include "slog.h"
@@ -74,7 +77,7 @@ static struct option long_options[] = {
 
 static struct server_data server_data;
 
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex;
 
 static int server_stop_requested;
 
@@ -254,15 +257,16 @@ create_response(const char *nurl, const char *method, unsigned int *rp_code)
 		free(fpath);
 	}
 
-	if (resp) {
+	if (resp)
 		return resp;
-	} else {
-		char *page = strdup(PAGE_NOT_FOUND);
-		*rp_code = MHD_HTTP_NOT_FOUND;
 
-		return MHD_create_response_from_data
-			(strlen(page), page, MHD_YES, MHD_NO);
-	}
+	char *page = strdup(PAGE_NOT_FOUND);
+	*rp_code = MHD_HTTP_NOT_FOUND;
+
+	return MHD_create_response_from_data(strlen(page),
+					     page,
+					     MHD_YES,
+					     MHD_NO);
 }
 
 static int cbk_http_request(void *cls,
@@ -298,9 +302,9 @@ static int cbk_http_request(void *cls,
 
 	nurl = url_normalize(url);
 
-	pthread_mutex_lock(&mutex);
+	pmutex_lock(&mutex);
 	response = create_response(nurl, method, &resp_code);
-	pthread_mutex_unlock(&mutex);
+	pmutex_unlock(&mutex);
 
 	ret = MHD_queue_response(connection, resp_code, response);
 	MHD_destroy_response(response);
@@ -388,11 +392,13 @@ int main(int argc, char *argv[])
 	if (!log_file)
 		log_file = strdup(DEFAULT_LOG_FILE);
 
+	pmutex_init(&mutex);
+
 	log_open(log_file);
 
-	psensor_init();
+	hddtemp_psensor_list_append(&server_data.sensors, 600);
 
-	server_data.sensors = get_all_sensors(0, 600);
+	lmsensor_psensor_list_append(&server_data.sensors, 600);
 
 #ifdef HAVE_GTOP
 	server_data.cpu_usage = create_cpu_usage_sensor(600);
@@ -426,17 +432,24 @@ int main(int argc, char *argv[])
 	}
 
 	while (!server_stop_requested) {
-		pthread_mutex_lock(&mutex);
+		pmutex_lock(&mutex);
 
 #ifdef HAVE_GTOP
 		sysinfo_update(&server_data.psysinfo);
 		cpu_usage_sensor_update(server_data.cpu_usage);
 #endif
-		psensor_list_update_measures(server_data.sensors);
+
+#ifdef HAVE_ATASMART
+		atasmart_psensor_list_update(server_data.sensors);
+#endif
+
+		hddtemp_psensor_list_update(server_data.sensors);
+
+		lmsensor_psensor_list_update(server_data.sensors);
 
 		psensor_log_measures(server_data.sensors);
 
-		pthread_mutex_unlock(&mutex);
+		pmutex_unlock(&mutex);
 		sleep(5);
 	}
 
@@ -454,7 +467,6 @@ int main(int argc, char *argv[])
 
 #ifdef HAVE_GTOP
 	sysinfo_cleanup();
-	cpu_cleanup();
 #endif
 
 	if (log_file != DEFAULT_LOG_FILE)
