@@ -96,7 +96,7 @@ static void update_menu_items(int use_celsius)
 	GtkMenuItem **m;
 
 	if (!sensors)
-		return ;
+		return;
 
 	for (s = sensors, m = menu_items; *s; s++, m++)
 		update_menu_item(*m, *s, use_celsius);
@@ -199,9 +199,9 @@ static void update_label(struct ui_psensor *ui)
 
 			if (is_temp_type((*p)->type))
 				str = "999UUU";
-			else if (is_fan_type((*p)->type))
+			else if ((*p)->type & SENSOR_TYPE_RPM)
 				str = "999UUU";
-			else /* cpu load */
+			else /* percent */
 				str = "999%";
 
 			if (guide == NULL) {
