@@ -18,6 +18,7 @@
  */
 #include <unity.h>
 
+#include <temperature.h>
 #include <ui_unity.h>
 
 static int initialized;
@@ -32,7 +33,7 @@ void ui_unity_launcher_entry_update(struct psensor **sensors,
 
 	if (!initialized) {
 		psensor_entry = unity_launcher_entry_get_for_desktop_file
-			("psensor.desktop");
+			(PSENSOR_DESKTOP_FILE);
 
 		unity_launcher_entry_set_count(psensor_entry, 0);
 		initialized = 1;
